
# $Id: 900-boilerplate.t,v 1.3 2009/04/14 12:09:17 aff Exp $

use strict;
use warnings;
use Test::More;

BEGIN {
  plan( skip_all => 'AUTHOR_TEST must be set for boilerplate test; skipping' )
    if ( !$ENV{'AUTHOR_TEST'} );
}

plan tests => 3;

sub not_in_file_ok {
  my ( $filename, %regex ) = @_;
  open( my $fh, '<', $filename )
    or die "couldn't open $filename for reading: $!";

  my %violated;

  while ( my $line = <$fh> ) {
    while ( my ( $desc, $regex ) = each %regex ) {
      if ( $line =~ $regex ) {
        push @{ $violated{$desc} ||= [] }, $.;
      }
    }
  }

  if ( scalar( keys %violated ) ) {
    fail( "$filename contains boilerplate text" );
    diag "$_ appears on lines @{$violated{$_}}" for keys %violated;
  } else {
    pass( "$filename contains no boilerplate text" );
  }
}

sub module_boilerplate_ok {
  my ( $module ) = @_;
  not_in_file_ok(
    $module => 'the great new $MODULENAME' => qr/ - The great new /,
    'boilerplate description'  => qr/Quick summary of what the module/,
    'stub function definition' => qr/function[12]/,
  );
}

not_in_file_ok(
  README => "The README is used..." => qr/The README is used/,
  "'version information here'" => qr/to provide version information/,
);

not_in_file_ok( Changes => "placeholder date/time" => qr(Date/time) );

module_boilerplate_ok( 'lib/Parse/Dia/SQL.pm' );

__END__
